
' *****************************************************************************
' *                                                                           *
' *     Ethernet.spin                                                         *
' *                                                                           *
' *****************************************************************************

' Known Bugs, Issues and Features
' ===============================
'
' Doesn't check / enforce maximum MTU_BYTES_MAX
' Doesn't check / enforce minimum MTU_BYTES_MIN
' Ethernet Packect Checksum CRC is bit-banged and could be improved
' Ethernet Packect Checksum CRC builds bits wrong way so need reversing
' Always forces an even number of bytes in UDP payload
' Uses Parllax Object TV_Text.spin to show the Ethernet Byte Packet
' TV_Text object can be deleted if not wanted
' PRI Display* routines can be deleted if not wanted
' PRI CreateFpgaDemoPacket creates a predefined, known packet. Can be deleted
'
' Hardware
' ========
'
' NOTE : ***** THIS HARDWARE HAS NOT BEEN TESTED ***** THEORETICAL ONLY *****
'
'   Propeller                                          View into RJ45 Socket
' .-----------.    2 x 68R     RJ45 Skt               .---------------------.
' |           |      ___                              | .-.-.-.-.-.-.-.-.-. |
' |    D+ P10 |-----|___|-----> Pin 1                 | | V V V V V V V V | |
' |           |      ___               To Hub         | | 1 2 3 4 5 6 7 8 | |
' |    D- P11 |-----|___|-----> Pin 2                 | `------.   .------' |
' |           |                                       |        `---'      ' |
' `-----------'    I = 25mA                           `---------------------'
'
' Pin    PC     Hub     T-568A            T-568B            Pairs
'
'  1     TX+    RX+     Green / White     Orange / White    ------.
'  2     TX-    RX-     Green             Orange            ------'
'  3     RX+    TX+     Orange / White    Green / White     ------.
'  4                    Blue              Blue              ---.  |
'  5                    Blue / White      Blue / White      ---'  |
'  6     RX-    TX-     Orange            Green             ------'
'  7                    Brown / White     Brown / White     ---.
'  8                    Brown             Brown             ---'
'
' Use straight-through cable to connect to Hub
' Use cross-over cable to connect to PC

CON

  CRC_POLYNOMIAL  = %0000_0100_1100_0001_0001_1101_1011_0111 ' $04C1_1DB7

  ' Array indexes into bytePacket[]
  
  ETH_PREAMBLE_0  = 0     ' 55
  ETH_PREAMBLE_1  = 1     ' 55
  ETH_PREAMBLE_2  = 2     ' 55
  ETH_PREAMBLE_3  = 3     ' 55
  ETH_PREAMBLE_4  = 4     ' 55
  ETH_PREAMBLE_5  = 5     ' 55
  ETH_PREAMBLE_6  = 6     ' 55
  
  ETH_SFD         = 7     ' D5

  ETH_DST_MAC_0   = 8     ' 00
  ETH_DST_MAC_1   = 9     ' 10
  ETH_DST_MAC_2   = 10    ' A4
  ETH_DST_MAC_3   = 11    ' 7B
  ETH_DST_MAC_4   = 12    ' EA
  ETH_DST_MAC_5   = 13    ' 80

  ETH_SRC_MAC_0   = 14    ' 00
  ETH_SRC_MAC_1   = 15    ' 12
  ETH_SRC_MAC_2   = 16    ' 34
  ETH_SRC_MAC_3   = 17    ' 56
  ETH_SRC_MAC_4   = 18    ' 78
  ETH_SRC_MAC_5   = 19    ' 90

  ETH_TYP_MSB     = 20    ' 08  IP
  ETH_TYP_LSB     = 21    ' 00

  IP_VER_IHL      = 22    ' 45
  IP_TOS          = 23    ' 00
  IP_LEN_MSB      = 24    ' 00
  IP_LEN_LSB      = 25    ' 2E
  IP_ID_MSB       = 26    ' B3
  IP_ID_LSB       = 27    ' FE
  IP_FRG_MSB      = 28    ' 00
  IP_FRG_LSB      = 29    ' 00
  IP_TTL          = 30    ' 80
  IP_PROTOCOL     = 31    ' 11  UDP
  IP_CHK_MSB      = 32    ' 05
  IP_CHK_LSB      = 33    ' 40

  IP_SRC_ADR_0    = 34    ' C0  192.168.0.44
  IP_SRC_ADR_1    = 35    ' A8
  IP_SRC_ADR_2    = 36    ' 00
  IP_SRC_ADR_3    = 37    ' 2C

  IP_DST_ADR_0    = 38    ' C0  192.168.0.4
  IP_DST_ADR_1    = 39    ' A8
  IP_DST_ADR_2    = 40    ' 00
  IP_DST_ADR_3    = 41    ' 04

  UDP_SRC_PRT_MSB = 42    ' 04  1024
  UDP_SRC_PRT_LSB = 43    ' 00

  UDP_DST_PRT_MSB = 44    ' 04  1024
  UDP_DST_PRT_LSB = 45    ' 00

  UDP_LEN_MSB     = 46    ' 00  18
  UDP_LEN_LSB     = 47    ' 1A

  UDP_CHK_MSB     = 48    ' 2D
  UDP_CHK_LSB     = 49    ' E8

  UDP_DAT_0       = 50    ' 00 m
  UDP_DAT_1       = 51    ' 01 m
  UDP_DAT_2       = 52    ' 02 m
  UDP_DAT_3       = 53    ' 03 m
  UDP_DAT_4       = 54    ' 04 m
  UDP_DAT_5       = 55    ' 05 m
  UDP_DAT_6       = 56    ' 06 p
  UDP_DAT_7       = 57    ' 07 p
  UDP_DAT_8       = 58    ' 08 p
  UDP_DAT_9       = 59    ' 09 p
  UDP_DAT_10      = 60    ' 0A n
  UDP_DAT_11      = 61    ' 0B n
  UDP_DAT_12      = 62    ' 0C n
  UDP_DAT_13      = 63    ' 0D n
  UDP_DAT_14      = 64    ' 0E
  UDP_DAT_15      = 65    ' 0F
  UDP_DAT_16      = 66    ' 10
  UDP_DAT_17      = 67    ' 11

  ETH_CHK_0       = 68    ' B3
  ETH_CHK_1       = 69    ' 31
  ETH_CHK_2       = 70    ' 88
  ETH_CHK_3       = 71    ' 1B

' *****************************************************************************
' *                                                                           *
' *
' *                                                                           *
' *****************************************************************************

OBJ

  Driver : "Ethernet_Driver"
  Tv     : "TV_Text"
  
' *****************************************************************************
' *                                                                           *
' *
' *                                                                           *
' *****************************************************************************

VAR

  byte bytePacket[ Driver#ETH_BYTES_MAX ]
  
  word udpLen
  word ipLen

  byte giveCrLf

  long packetNumber
  
' *****************************************************************************
' *                                                                           *
' *
' *                                                                           *
' *****************************************************************************

PUB Start( ethernetBasePin, tvBasePin )

  Tv.Start( tvBasePin )
  
  Driver.Start( ethernetbasePin )

PUB CreateEmptyUdpPacket

  packetNumber++
  
  bytePacket[ ETH_PREAMBLE_0  ] := $55
  bytePacket[ ETH_PREAMBLE_1  ] := $55
  bytePacket[ ETH_PREAMBLE_2  ] := $55
  bytePacket[ ETH_PREAMBLE_3  ] := $55
  bytePacket[ ETH_PREAMBLE_4  ] := $55
  bytePacket[ ETH_PREAMBLE_5  ] := $55
  bytePacket[ ETH_PREAMBLE_6  ] := $55
  
  bytePacket[ ETH_SFD         ] := $D5

  bytePacket[ ETH_TYP_MSB     ] := $08    
  bytePacket[ ETH_TYP_LSB     ] := $00

  bytePacket[ IP_VER_IHL      ] := $45
  bytePacket[ IP_TOS          ] := $00
  bytePacket[ IP_ID_MSB       ] := packetNumber >> 8
  bytePacket[ IP_ID_LSB       ] := packetNumber
  bytePacket[ IP_FRG_MSB      ] := $00
  bytePacket[ IP_FRG_LSB      ] := $00
  bytePacket[ IP_TTL          ] := $80
  bytePacket[ IP_PROTOCOL     ] := $11
                                             
  ipLen := 28

  bytePacket[ IP_LEN_MSB      ] := ipLen >> 8
  bytePacket[ IP_LEN_LSB      ] := ipLen
               
  udpLen := 8

  bytePacket[ UDP_LEN_MSB     ] := udpLen >> 8
  bytePacket[ UDP_LEN_LSB     ] := udpLen

PUB SetDstMac(mac0,mac1,mac2,mac3,mac4,mac5)

  bytePacket[ ETH_DST_MAC_0 ] := mac0
  bytePacket[ ETH_DST_MAC_1 ] := mac1
  bytePacket[ ETH_DST_MAC_2 ] := mac2
  bytePacket[ ETH_DST_MAC_3 ] := mac3
  bytePacket[ ETH_DST_MAC_4 ] := mac4
  bytePacket[ ETH_DST_MAC_5 ] := mac5

PUB SetDstAdr(adr0,adr1,adr2,adr3)

  bytePacket[ IP_DST_ADR_0 ] := adr0
  bytePacket[ IP_DST_ADR_1 ] := adr1
  bytePacket[ IP_DST_ADR_2 ] := adr2
  bytePacket[ IP_DST_ADR_3 ] := adr3

PUB SetDstPrt(port)

  bytePacket[ UDP_DST_PRT_MSB ] := port >> 8
  bytePacket[ UDP_DST_PRT_LSB ] := port 

PUB SetSrcMac(mac0,mac1,mac2,mac3,mac4,mac5)

  bytePacket[ ETH_SRC_MAC_0 ] := mac0
  bytePacket[ ETH_SRC_MAC_1 ] := mac1
  bytePacket[ ETH_SRC_MAC_2 ] := mac2
  bytePacket[ ETH_SRC_MAC_3 ] := mac3
  bytePacket[ ETH_SRC_MAC_4 ] := mac4
  bytePacket[ ETH_SRC_MAC_5 ] := mac5

PUB SetSrcAdr(adr0,adr1,adr2,adr3)

  bytePacket[ IP_SRC_ADR_0 ] := adr0
  bytePacket[ IP_SRC_ADR_1 ] := adr1
  bytePacket[ IP_SRC_ADR_2 ] := adr2
  bytePacket[ IP_SRC_ADR_3 ] := adr3

PUB SetSrcPrt(port)

  bytePacket[ UDP_SRC_PRT_MSB ] := port >> 8
  bytePacket[ UDP_SRC_PRT_LSB ] := port 

PUB AddUdpData(data)
          
  bytePacket[ UDP_DAT_0 + udpLen - 8 ] := data

  udpLen++

  bytePacket[ UDP_LEN_MSB ] := udpLen >> 8
  bytePacket[ UDP_LEN_LSB ] := udpLen
               
  ipLen++

  bytePacket[ IP_LEN_MSB ] := ipLen >> 8
  bytePacket[ IP_LEN_LSB ] := ipLen

PUB GetBytePacketWord( msbOffset, lsbOffset )

  result := ( bytePacket[ msbOffset ] << 8 ) | bytePacket[ lsbOffset ]

PUB GetBytePacketByte( offset )

  result := bytePacket[ offset ]

PUB GetPacketNumber

  result := packetNumber
                 
' *****************************************************************************
' *                                                                           *
' *
' *                                                                           *
' *****************************************************************************

PUB SendUdpPacket

  'CreateFpgaDemoPacket

  PadUdpData
  FixupUdpChecksum
  FixupIpHeaderChecksum
  FixupPacketChecksum

  DisplayBytePacket
  'DisplayBitArray
  
  Driver.CreateAndSendBitStream( @bytePacket )

PRI PadUdpData

  if ( udpLen & 1 ) ' UDP must be even number of bytes ( ie words )
    AddUdpData(0)

PRI FixupIpHeaderChecksum | checksum ' As per RFC 791

  checksum := 0

  ' Add in the IP Header
  
  checksum += GetBytePacketWord( IP_VER_IHL     , IP_TOS          )          
  checksum += GetBytePacketWord( IP_LEN_MSB     , IP_LEN_LSB      )
  checksum += GetBytePacketWord( IP_ID_MSB      , IP_ID_LSB       )
  checksum += GetBytePacketWord( IP_FRG_MSB     , IP_FRG_LSB      )
  checksum += GetBytePacketWord( IP_TTL         , IP_PROTOCOL     )  
  checksum += GetBytePacketWord( IP_SRC_ADR_0   , IP_SRC_ADR_1    )
  checksum += GetBytePacketWord( IP_SRC_ADR_2   , IP_SRC_ADR_3    )
  checksum += GetBytePacketWord( IP_DST_ADR_0   , IP_DST_ADR_1    )
  checksum += GetBytePacketWord( IP_DST_ADR_2   , IP_DST_ADR_3    )

  ' Adjust to be 16-bit, one's complement

  repeat ( checksum >> 16 )
    checksum := ( checksum >> 16 ) + ( checksum & $FFFF ) 
  checksum ^= $FFFF
  if checksum == $0000
    checksum := $FFFF

  ' Update the packet
  
  bytePacket[ IP_CHK_MSB ] := checksum >> 8
  bytePacket[ IP_CHK_LSB ] := checksum 

PRI FixupUdpChecksum | checksum, i ' As per RFC 768

  checksum := 0
  
  ' Add in Psuedo IP Header

  checksum += GetBytePacketWord( IP_SRC_ADR_0   , IP_SRC_ADR_1    )
  checksum += GetBytePacketWord( IP_SRC_ADR_2   , IP_SRC_ADR_3    )
  checksum += GetBytePacketWord( IP_DST_ADR_0   , IP_DST_ADR_1    )
  checksum += GetBytePacketWord( IP_DST_ADR_2   , IP_DST_ADR_3    )
  checksum += GetBytePacketByte(                  IP_PROTOCOL     )
  checksum += GetBytePacketWord( UDP_LEN_MSB    , UDP_LEN_LSB     )

  ' Add in UDP Header

  checksum += GetBytePacketWord( UDP_SRC_PRT_MSB, UDP_SRC_PRT_LSB )
  checksum += GetBytePacketWord( UDP_DST_PRT_MSB, UDP_DST_PRT_LSB )
  checksum += GetBytePacketWord( UDP_LEN_MSB    , UDP_LEN_LSB     )

  ' Add in UDP Payload

  repeat i from 0 to ( udpLen - 8 - 1) step 2
    checksum += GetBytePacketWord( UDP_DAT_0 + i, UDP_DAT_1 + i )

  ' Adjust to be 16-bit, one's complement

  repeat ( checksum >> 16 )
    checksum := ( checksum >> 16 ) + ( checksum & $FFFF ) 
  checksum ^= $FFFF
  if checksum == $0000
    checksum := $FFFF

  ' Update the packet
  
  bytePacket[ UDP_CHK_MSB ] := checksum >> 8
  bytePacket[ UDP_CHK_LSB ] := checksum 

PRI FixupPacketChecksum | checksum,i

  checksum := $FFFF_FFFF

  repeat i from 8 to UDP_DAT_0 + udpLen - 8 - 1
    checksum := CrcAddByte( checksum, bytePacket[i] ) 

  checksum := ( checksum >< 32 ) ^ $FFFF_FFFF

  bytePacket[ UDP_DAT_0 + udpLen - 8  ] := checksum 
  bytePacket[ UDP_DAT_1 + udpLen - 8  ] := checksum >> 8
  bytePacket[ UDP_DAT_2 + udpLen - 8  ] := checksum >> 16
  bytePacket[ UDP_DAT_3 + udpLen - 8  ] := checksum >> 24
 
PRI CrcAddByte( crc, n ) | k
  
  repeat 8
    k := ( n  ^ ( crc >> 31 ) ) &  1
    if k
      k := CRC_POLYNOMIAL
    crc := ( crc << 1 ) ^ k
    n >>= 1
  result := crc

' *****************************************************************************
' *                                                                           *
' *
' *                                                                           *
' *****************************************************************************

PRI DisplayBytePacket

  tv.out(1) ' Home display
  giveCrLf := false
   
  DisplayData( String("ETH PRE"), ETH_PREAMBLE_0         ,  8  )
  DisplayData( String("DST MAC"), ETH_DST_MAC_0          ,  6  )
  DisplayData( String("SRC MAC"), ETH_SRC_MAC_0          ,  6  )
  DisplayData( String("ETH TYP"), ETH_TYP_MSB            ,  2  )
  DisplayData( String("IP  VER"), IP_VER_IHL             , -1  )
  DisplayData( String("IP  TOS"), IP_TOS                 ,  1  )
  DisplayData( String("IP  LEN"), IP_LEN_MSB             , -2  )
  DisplayData( String("IP  ID "), IP_ID_MSB              ,  2  )
  DisplayData( String("IP  FRG"), IP_FRG_MSB             , -2  )
  DisplayData( String("IP  TTL"), IP_TTL                 ,  1  )
  DisplayData( String("IP  PRO"), IP_PROTOCOL            , -1  )
  DisplayData( String("IP  CHK"), IP_CHK_MSB             ,  2  )
  DisplayData( String("SRC ADR"), IP_SRC_ADR_0           ,  4  )
  DisplayData( String("DST ADR"), IP_DST_ADR_0           , -4  )
  DisplayData( String("SRC PRT"), UDP_SRC_PRT_MSB        ,  2  )
  DisplayData( String("DST PRT"), UDP_DST_PRT_MSB        , -2  )
  DisplayData( String("UDP LEN"), UDP_LEN_MSB            ,  2  )
  DisplayData( String("UDP CHK"), UDP_CHK_MSB            , -2  )
  DisplayData( String("UDP DAT"), UDP_DAT_0              ,  10 )
  DisplayData( String("      +"), UDP_DAT_10             ,  4  )
  DisplayData( String("ETH CHK"), UDP_DAT_0 + udpLen - 8 , -4  )
  
PRI DisplayData( txtPtr, offset, count )

  if count < 0
    tv.out( $A )
    tv.out( 20 )
    count := || count
  else
    if giveCrLf 
      tv.out( $D )
  tv.out( $C )
  tv.out( 2 )
  tv.str( txtPtr )
  tv.out( $C )
  tv.out( 1 )
  repeat count--
    tv.out( " " )
    tv.hex( bytePacket[offset++], 2 )
  giveCrLf := true

PRI DisplayBitArray | pointerToBitArray, i, n

    pointerToBitArray := Driver.GetPointerToBitArray

    tv.out(1)
    repeat i from 1 to 13
      if i <> 1
        tv.out($D)
      tv.out($C)
      tv.out(1)
      tv.hex( i, 1)
      tv.out(" ")
      tv.out($C)
      tv.out(0)
      DisplayBits( long[ pointerToBitArray+(i*4) ] )

PRI DisplayBits( n )

  tv.bin( n >> 24, 8 )
  tv.out( " " )
  tv.bin( n >> 16, 8 )
  tv.out( " " )
  tv.bin( n >> 8,  8 )
  tv.out( " " )
  tv.bin( n,       8 )
  tv.out( " " )
  tv.out( ">" )
 
' *****************************************************************************
' *                                                                           *
' *
' *                                                                           *
' *****************************************************************************

PRI CreateFpgaDemoPacket

' Ethernet preamble:            55 55 55 55 55 55 55
' Ethernet SFD:                 D5
' Ethernet destination address: 00 10 A4 7B EA 80
' Ethernet source address:      00 12 34 56 78 90
' Ethernet type:                08 00 (=IP)
' IP header:                    45 00 00 2E B3 FE 00 00 80
' IP protocol:                  11 (=UDP)
' IP checksum:                  05 40
' IP source (192.168.0.44):     C0 A8 00 2C
' IP destination (192.168.0.4): C0 A8 00 04
' UDP source port (1024):       04 00
' UDP destination port (1024):  04 00
' UDP payload length (18):      00 1A
' UDP checksum:                 2D E8
' UDP payload (18 bytes):       00 01 02 03 04 05 06 07 08 09 0A 0B 0C 0D 0E 0F 10 11
' Ethernet checksum:            B3 31 88 1B

  bytePacket[ ETH_PREAMBLE_0  ] := $55
  bytePacket[ ETH_PREAMBLE_1  ] := $55
  bytePacket[ ETH_PREAMBLE_2  ] := $55
  bytePacket[ ETH_PREAMBLE_3  ] := $55
  bytePacket[ ETH_PREAMBLE_4  ] := $55
  bytePacket[ ETH_PREAMBLE_5  ] := $55
  bytePacket[ ETH_PREAMBLE_6  ] := $55
  bytePacket[ ETH_SFD         ] := $D5

  bytePacket[ ETH_DST_MAC_0   ] := $00
  bytePacket[ ETH_DST_MAC_1   ] := $10
  bytePacket[ ETH_DST_MAC_2   ] := $A4
  bytePacket[ ETH_DST_MAC_3   ] := $7B
  bytePacket[ ETH_DST_MAC_4   ] := $EA
  bytePacket[ ETH_DST_MAC_5   ] := $80

  bytePacket[ ETH_SRC_MAC_0   ] := $00
  bytePacket[ ETH_SRC_MAC_1   ] := $12
  bytePacket[ ETH_SRC_MAC_2   ] := $34
  bytePacket[ ETH_SRC_MAC_3   ] := $56
  bytePacket[ ETH_SRC_MAC_4   ] := $78
  bytePacket[ ETH_SRC_MAC_5   ] := $90

  bytePacket[ ETH_TYP_MSB     ] := $08    
  bytePacket[ ETH_TYP_LSB     ] := $00

  bytePacket[ IP_VER_IHL      ] := $45
  bytePacket[ IP_TOS          ] := $00
  bytePacket[ IP_LEN_MSB      ] := $00
  bytePacket[ IP_LEN_LSB      ] := $2E
  bytePacket[ IP_ID_MSB       ] := $B3
  bytePacket[ IP_ID_LSB       ] := $FE
  bytePacket[ IP_FRG_MSB      ] := $00
  bytePacket[ IP_FRG_LSB      ] := $00
  bytePacket[ IP_TTL          ] := $80
  bytePacket[ IP_PROTOCOL     ] := $11
  bytePacket[ IP_CHK_MSB      ] := $05
  bytePacket[ IP_CHK_LSB      ] := $40

  bytePacket[ IP_SRC_ADR_0    ] := $C0
  bytePacket[ IP_SRC_ADR_1    ] := $A8
  bytePacket[ IP_SRC_ADR_2    ] := $00
  bytePacket[ IP_SRC_ADR_3    ] := $2C

  bytePacket[ IP_DST_ADR_0    ] := $C0
  bytePacket[ IP_DST_ADR_1    ] := $A8
  bytePacket[ IP_DST_ADR_2    ] := $00
  bytePacket[ IP_DST_ADR_3    ] := $04

  bytePacket[ UDP_SRC_PRT_MSB ] := $04
  bytePacket[ UDP_SRC_PRT_LSB ] := $00

  bytePacket[ UDP_DST_PRT_MSB ] := $04
  bytePacket[ UDP_DST_PRT_LSB ] := $00

  bytePacket[ UDP_LEN_MSB     ] := $00
  bytePacket[ UDP_LEN_LSB     ] := $1A

  bytePacket[ UDP_CHK_MSB     ] := $2D
  bytePacket[ UDP_CHK_LSB     ] := $E8

  bytePacket[ UDP_DAT_0       ] := $00
  bytePacket[ UDP_DAT_1       ] := $01
  bytePacket[ UDP_DAT_2       ] := $02
  bytePacket[ UDP_DAT_3       ] := $03
  bytePacket[ UDP_DAT_4       ] := $04
  bytePacket[ UDP_DAT_5       ] := $05
  bytePacket[ UDP_DAT_6       ] := $06
  bytePacket[ UDP_DAT_7       ] := $07
  bytePacket[ UDP_DAT_8       ] := $08
  bytePacket[ UDP_DAT_9       ] := $09
  bytePacket[ UDP_DAT_10      ] := $0A
  bytePacket[ UDP_DAT_11      ] := $0B
  bytePacket[ UDP_DAT_12      ] := $0C
  bytePacket[ UDP_DAT_13      ] := $0D
  bytePacket[ UDP_DAT_14      ] := $0E
  bytePacket[ UDP_DAT_15      ] := $0F
  bytePacket[ UDP_DAT_16      ] := $10
  bytePacket[ UDP_DAT_17      ] := $11

  bytePacket[ ETH_CHK_0       ] := $B3
  bytePacket[ ETH_CHK_1       ] := $31
  bytePacket[ ETH_CHK_2       ] := $88
  bytePacket[ ETH_CHK_3       ] := $1B

