
' *****************************************************************************
' *                                                                           *
' *     Ethernet_Demo.spin                                                    *
' *                                                                           *
' *****************************************************************************

' Known Bugs, Issues and Features
' ===============================
'
' Requires 80MHz CLKFREQ
' TV_BASE_PIN set to P12..P14. May need changing
' ETHERNET_BASE_PIN set for P10 (D+) and P11 (D-). May need changing
' Parallax's TV_Text.spin uses NTSC. May need changing for PAL users
'
' Hardware
' ========
'
' NOTE : ***** THIS HARDWARE HAS NOT BEEN TESTED ***** THEORETICAL ONLY *****
'
'   Propeller                                          View into RJ45 Socket
' .-----------.    2 x 68R     RJ45 Skt               .---------------------.
' |           |      ___                              | .-.-.-.-.-.-.-.-.-. |
' |    D+ P10 |-----|___|-----> Pin 1                 | | V V V V V V V V | |
' |           |      ___               To Hub         | | 1 2 3 4 5 6 7 8 | |
' |    D- P11 |-----|___|-----> Pin 2                 | `------.   .------' |
' |           |                                       |        `---'      ' |
' `-----------'    I = 25mA                           `---------------------'
'
' Pin    PC     Hub     T-568A            T-568B            Pairs
'
'  1     TX+    RX+     Green / White     Orange / White    ------.
'  2     TX-    RX-     Green             Orange            ------'
'  3     RX+    TX+     Orange / White    Green / White     ------.
'  4                    Blue              Blue              ---.  |
'  5                    Blue / White      Blue / White      ---'  |
'  6     RX-    TX-     Orange            Green             ------'
'  7                    Brown / White     Brown / White     ---.
'  8                    Brown             Brown             ---'
'
' Use straight-through cable to connect to Hub
' Use cross-over cable to connect to PC

CON

  _CLKMODE          = XTAL1 + PLL16x
  _XINFREQ          = 5_000_000

  TV_BASE_PIN       = 12
  ETHERNET_BASE_PIN = 10

  UDP_UNIT          = 1                                 ' Must be unique for every source ( 0..255 )
  UDP_PORT          = 1024                              ' Must be same as receiver
  
OBJ

  Ethernet : "Ethernet"
  
PUB Main | packetNumber

  Ethernet.Start( ETHERNET_BASE_PIN , TV_BASE_PIN )

  ' Set Source
  
  ' Source MAC Address = $AA $BB $CC $DD $EE $FF
  '                      -----.----- -----.-----
  '                           |           |
  '                           |           `-------------- Serial Number
  '                           |
  '                           `-------------------------- Vendor / Manufacturer's ID ( OUI )
  '
  '                      %AAAAAAAA
  '                             ||
  '                             |`----------------------- Group / Multicast Bit ( for source must be 0 )
  '                             `------------------------ 1 = Locally Administered / 0 = Burned-In
  
  Ethernet.SetSrcMac($02,$00,$00,$00,$00,UDP_UNIT)      ' Locally Administered MAC Address
  Ethernet.SetSrcAdr(169,254,0,UDP_UNIT)                ' Auto-Configured IP Address ( RFC 3330 )                      ' 
  Ethernet.SetSrcPrt(UDP_PORT)                          ' Port

 ' Set Destination - Broadcast to Port 1024

  Ethernet.SetDstMac($FF,$FF,$FF,$FF,$FF,$FF)           ' Broadcast MAC Address
  Ethernet.SetDstAdr(255,255,255,255)                   ' Broadcast IP Address
  Ethernet.SetDstPrt(UDP_PORT)                          ' Port
  
  repeat  

    Ethernet.CreateEmptyUdpPacket                       ' Start a new UDP Packet

    ' Put some meaningful data into the UDP payload ...
    
    Ethernet.AddUdpData( Ethernet.GetBytePacketByte( Ethernet#ETH_SRC_MAC_0 ) )
    Ethernet.AddUdpData( Ethernet.GetBytePacketByte( Ethernet#ETH_SRC_MAC_1 ) )
    Ethernet.AddUdpData( Ethernet.GetBytePacketByte( Ethernet#ETH_SRC_MAC_2 ) )
    Ethernet.AddUdpData( Ethernet.GetBytePacketByte( Ethernet#ETH_SRC_MAC_3 ) )
    Ethernet.AddUdpData( Ethernet.GetBytePacketByte( Ethernet#ETH_SRC_MAC_4 ) )
    Ethernet.AddUdpData( Ethernet.GetBytePacketByte( Ethernet#ETH_SRC_MAC_5 ) )

    Ethernet.AddUdpData( Ethernet.GetBytePacketByte( Ethernet#IP_SRC_ADR_0  ) )
    Ethernet.AddUdpData( Ethernet.GetBytePacketByte( Ethernet#IP_SRC_ADR_1  ) )
    Ethernet.AddUdpData( Ethernet.GetBytePacketByte( Ethernet#IP_SRC_ADR_2  ) )
    Ethernet.AddUdpData( Ethernet.GetBytePacketByte( Ethernet#IP_SRC_ADR_3  ) )

    ' An incrementing count always helps things along ...

    packetNumber := Ethernet.GetPacketNumber
    
    Ethernet.AddUdpData( packetNumber >> 24 )            
    Ethernet.AddUdpData( packetNumber >> 16 )
    Ethernet.AddUdpData( packetNumber >> 8  )
    Ethernet.AddUdpData( packetNumber       )

    Ethernet.SendUdpPacket                              ' Send the Packet

    PauseMs( 1000 )                                     ' Send one per second

PRI PauseMs( mS )

  waitcnt( clkfreq / 1000 * mS + CNT )
  